#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# dataport-schuposh-twofactor-admin-module
# listener module
#
# Copyright 2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.
#
# README:
#
# This module watches changes of the group "admins-$OU"
# and adds members of this group to "mfa-admins".
# All group changes are collected and after 15 seconds of "silence", the
# required group modifications are performed only once per OU.

import listener
import univention.debug
import univention.admin.uldap


name = "dataport-schuposh-twofactor-admin-module"
description = "Trigger update of group mfa-admins upon school admin group change"
filter = (
    "(&(|(ucsschoolRole=school_admin_group:school:*))(objectClass=univentionGroup))"
)

ldap_base = listener.configRegistry["ldap/base"]
domain_name = listener.configRegistry["domainname"]
server_role = listener.configRegistry["server/role"]

ADMINS_PREFIX = (
    listener.configRegistry["ucsschool/ldap/default/groupprefix/admins"]
).encode()
if not ADMINS_PREFIX:
    ADMINS_PREFIX = b"admins-"

do_refresh = False


def handler(dn, new, old):
    global do_refresh

    if server_role != "domaincontroller_master":
        return

    # get common name of old or new object
    cn = new.get("cn", [None])[0] or old.get("cn", [None])[0]
    if not cn:
        # should not happen :-)
        univention.debug.debug(
            univention.debug.LISTENER,
            univention.debug.ERROR,
            "{}: cn is not set: {}".format(name, dn),
        )
        return

    # if group prefix matches, add item to list of pending OUs
    if cn.startswith(ADMINS_PREFIX):
        univention.debug.debug(
            univention.debug.LISTENER,
            univention.debug.PROCESS,
            '{}: triggered by group "{}"'.format(name, cn),
        )
        do_refresh = True


def postrun():  # type: () -> None
    global do_refresh

    cmd = ["/usr/sbin/update-mfa-admin-group", "update"]

    univention.debug.debug(
        univention.debug.LISTENER,
        univention.debug.PROCESS,
        '{}: calling "{}"'.format(name, " ".join(cmd)),
    )
    # "listener.run" expects argv to be a tuple or list
    listener.run(cmd[0], [cmd[1]], uid=0)

    do_refresh = False

# vim: set et sw=4 ts=4
