/*
 * Copyright 2011-2022 Univention GmbH
 *
 * https://www.univention.de/
 *
 * All rights reserved.
 *
 * The source code of this program is made available
 * under the terms of the GNU Affero General Public License version 3
 * (GNU AGPL V3) as published by the Free Software Foundation.
 *
 * Binary versions of this program provided by Univention to you as
 * well as other copyrighted, protected or trademarked materials like
 * Logos, graphics, fonts, specific documentations and configurations,
 * cryptographic keys etc. are subject to a license agreement between
 * you and Univention and not subject to the GNU AGPL V3.
 *
 * In the case you use this program under the terms of the GNU AGPL V3,
 * the program is provided in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License with the Debian GNU/Linux or Univention distribution in file
 * /usr/share/common-licenses/AGPL-3; if not, see
 * <https://www.gnu.org/licenses/>.
 */
/*global define*/

define([
    "umc/dialog",
    "umc/widgets/Form",
    "umc/widgets/Button",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "umc/modules/twofactor_admin/UserGridChooseSchool",
    "umc/widgets/Module",
    "umc/widgets/Image",
    "umc/widgets/Page",
    "umc/widgets/Grid",
    "umc/widgets/SearchForm",
    "umc/widgets/ComboBox",
    "umc/widgets/SearchBox",
    "umc/widgets/TextBox",
    "dojo/_base/array",
    "umc/tools",
    "umc/i18n!umc/modules/twofactor_admin"
], function (dialog, Form, Button, declare, lang, UserGridChooseSchool, Module, Image, Page, Grid, SearchForm, ComboBox, SearchBox, TextBox, array,  tools, _) {
    return declare("umc.modules.twofactor_admin", [Module], {

        _grid: null,
        _store: null,
        _searchWidget: null,
        _contextVariable: null,
        _page: null,
        _form: null,
        _schools: null,
        _currentSchool: null,
        _choosePage: null,
        _hardwareTokenDialog: null,
        _headerButtons: null,
        idProperty: 'uid',

        getHardwareTokenForm(uid, handler) {
            var buttons = [{
                type: Button,
                label: _("Cancel"),
                name: 'hardwaretoken_cancel',
                onClick: lang.hitch(this, function () {
                    this._hardwareTokenDialog.hide()
                    this._hardwareTokenDialog.destroy()
                })
            },
                {
                    type: Button,
                    label: _("Submit"),
                    onClick: uid ? lang.hitch(this, handler, uid) : lang.hitch(this, handler),
                    name: 'hardwaretoken_submit'
                }]

            var form = new Form({
                widgets: [{
                    type: TextBox,
                    constraints: {pattern: "#"},
                    name: 'serial',
                    label: _('Serial Number'),
                    required: 'true',
                    'data-qa': 'serialnumber'
                },
                    {
                        type: TextBox,
                        name: 'current_otp',
                        constraints: {pattern: "#"},
                        label: _('Current OTP (number on Token device display)'),
                        required: 'true',
                        'data-qa': 'current_otp',
                        regExp: '\\d{6}'
                    }],
                buttons: buttons
            });

            return form
        },


        assignHardwareToken: function (uid) {
            var form = this._hardwareTokenDialog.content

            if (!(form.getWidget("serial").isValid() && form.getWidget("current_otp").isValid())) return
            var values = form.getValues()
            var p = {'uid': uid[0], 'serial': values['serial'], 'current_otp_value': values['current_otp']}
            this.standbyDuring(this.umcpCommand("twofactor_admin/assign_hardware_token", p)).then(lang.hitch(this, function (ret) {
                if (ret['result'] == "success") {
                    dialog.confirm(_("Token was assigned successfully"),
                        [{
                            'default': true,
                            label: _('OK')
                        }]
                    )
                    this._hardwareTokenDialog.hide()
                    this._hardwareTokenDialog.destroy()
                }
                else {
                    dialog.confirm(_("Error assigning Hardware Token"),
                        [{
                            'default': true,
                            label: _('OK')
                        }]
                    )
                }
            }))

        },

        rollOverHardwareToken: function (uid) {
            this._hardwareTokenDialog = new dialog({
                title: _("Assign Hardware Token"),
                content: this.getHardwareTokenForm(uid, this.assignHardwareToken),
                'class': 'umcPopup'
            })
            this._hardwareTokenDialog.show()
        },



        rolloverQrToken: function (uid) {
            var params = {
                uid: uid[0]
            };
            var msg;


            msg = _('ALL other Tokens of the user will be replaced by the new Token', uid);

            dialog.confirm(msg, [{
                'default': true,
                label: _('Cancel')
            }, {
                label: _('OK'),
                callback: lang.hitch(this, function () {
                    this.standbyDuring(this.umcpCommand("twofactor_admin/rollover_qr_token", params)).then(lang.hitch(this, function (ret) {
                        console.log(ret)
                        if (! ret['result'].hasOwnProperty("image")) {
                            dialog.confirm(_("Error fetching QR-Code Image"),
                                [{
                                    'default': true,
                                    label: _('OK')
                                }]
                            )
                        } else {

                            var form = new Form();
                            var qrcode = new Image({
                                id: "qrcode",
                                value: ret['result']['image'].replace("data:image/png;base64,", ""),
                                style: "background: none; max-width:100%; max-height:100%;",
                                'data-qa': 'qrcode-image'
                            })

                            qrcode.placeAt(form.containerNode)

                            new Button({
                                label: _("Close"),
                                'data-qa': 'closebutton',
                                'class': 'ucsTextButton',
                                onClick: lang.hitch(this, function () {
                                    qrcode.destroy()
                                    myDialog.hide()
                                })
                            }, "progbutton").placeAt(form.containerNode);

                            new Button({
                                label: _("Print"),
                                //'class': 'ucsTextButton',
                                'data-qa': "printbutton",
                                onClick: lang.hitch(this, function () {
                                    var printContents = document.getElementById('qrcode').innerHTML
                                    w = window.open();
                                    w.document.write(printContents);
                                    w.print();
                                    w.close();
                                })
                            }, "progbutton").placeAt(form.containerNode);

                            var myDialog = new dialog({
                                title: _("Please ask the user to scan the QR-Code."),
                                style: "width: 100%",
                                content: form
                            });

                            myDialog.show()
                            this.reloadGrid();
                        }
                    }));
                })
            }]);
        },

        lostToken: function (uid) {
            var params = {
                uid: uid[0]
            };
            var msg;

            msg = _('ALL existing Tokens of the user will be deleted or disabled and unassigned (in case of hardware token).', uid);

            dialog.confirm(msg, [{
                'default': true,
                label: _('Cancel')
            }, {
                label: _('OK'),
                callback: lang.hitch(this, function () {
                    this.standbyDuring(this.umcpCommand("twofactor_admin/lost_token", params)).then(lang.hitch(this, function (ret) {
                        switch (ret.result) {
                            case "success":
                                dialog.confirm(_("Token was reported lost/broken successfully"),
                                    [{
                                        'default': true,
                                        label: _('OK')
                                    }]
                                )
                                break;
                            default:
                                dialog.confirm(_("Error during reporting of lost/broken Token"),
                                    [{
                                        'default': true,
                                        label: _('OK')
                                    }]
                                )

                        }
                    }))
                })
            }])

        },

        reloadGrid: function () {
            this._grid.filter(this._grid.query);
        },

        showTokenGrid: function (schoolSelection) {
            // find current school and set it
            this._currentSchool = array.filter(this._schools, lang.hitch(this, function (school) {
                return school.id === schoolSelection;
            }))[0]
            // set current school on server side
            this.standbyDuring(this.umcpCommand("twofactor_admin/set_current_school", {school: this._currentSchool.id})).then(lang.hitch(this, function () {
                this.tokenSelfService();
            }))
        },

        returnHardwareTokenAction: function (serial, otp) {
            var form = this._hardwareTokenDialog.content
            //var current_otp = form.getWidget("current_otp")
            //var valid = current_otp.isValid()
            if (!(form.getWidget("serial").isValid() && form.getWidget("current_otp").isValid())) return
            var values = form.getValues()
            var p = {'serial': values['serial'], 'current_otp_value': values['current_otp']}
            this.standbyDuring(this.umcpCommand("twofactor_admin/return_hardware_token", p)).then(lang.hitch(this, function (ret) {
                switch (ret.result) {
                    case "success":
                        dialog.confirm(_("Token was returned successfully"),
                            [{
                                'default': true,
                                label: _('OK')
                            }]
                        )
                        this._hardwareTokenDialog.hide()
                        this._hardwareTokenDialog.destroy()
                        break;
                    default:
                        dialog.confirm(_( ret.result),
                            [{
                                'default': true,
                                label: _('OK')
                            }]
                        )
                        break;
                }
            }))
        },

        returnHardwareToken: function () {
            this._hardwareTokenDialog = new dialog({
                title: _("Return Hardware Token"),
                content: this.getHardwareTokenForm(null, this.returnHardwareTokenAction),
                'class': 'umcPopup'
            })
            this._hardwareTokenDialog.show()
        },

        tokenSelfService: function () {
            this._headerButtons = new Button({
                label: _("Return Hardware Token"),
                'data-qa': 'return__hardware_token',
                onClick: lang.hitch(this, this.returnHardwareToken)
            })

            this._page = new Page({
                helpText: _('Select an entry and press a button according to the desired token type'),
                fullWidth: true,
            });
            this.addChild(this._page);

            var actions = [{
                name: 'rollover_otptoken',
                label: _('QR-Code ("Handy") Token'),
                callback: lang.hitch(this, 'rolloverQrToken'),
                isStandardAction: true,
                isMultiAction: false,
            }, {
                name: 'rollout_htoptoken',
                label: _('Hardware Token'),
                callback: lang.hitch(this, 'rollOverHardwareToken'),
                isStandardAction: true,
                isMultiAction: false
            }, {
                name: 'lost_token',
                label: _('Lost Token'),
                callback: lang.hitch(this, 'lostToken'),
                isStandardAction: true,
                isMultiAction: false
            }];


            var columns = [{
                name: 'uid',
                label: _('User'),
                width: '100px'
            }, {
                name: 'firstname',
                label: _('Firstname'),
                width: '70px'
            }, {
                name: 'lastname',
                label: _('Lastname'),
                width: 'adjust',
            }];

            this._grid = new Grid({
                region: 'main',
                actions: actions,
                columns: columns,
                moduleStore: this.moduleStore,
                sortIndex: -3,
            });

            // setting via constructor triggers autosearch
            this._grid.query = {
                category: 'all',
                pattern: ''
            }

            var widgets = [{
                type: SearchBox,
                name: 'pattern',
                value: '',
                inlineLabel: _('Search...'),
                onSearch: lang.hitch(this, function () {
                    this._searchWidget.submit();
                })
            }];

            var searchForm = new SearchForm({
                region: 'nav',
                hideSubmitButton: true,
                widgets: widgets,
                layout: ['pattern'],
                onSearch: lang.hitch(this._grid, 'filter')
            })

            this._searchWidget = searchForm
            this._page.addChild(this._searchWidget);
            this._page.addChild(this._headerButtons);
            this._page.addChild(this._grid);
            this.selectChild(this._page)

            tools.ucr(['twofactor_admin/autosearch']).then(lang.hitch(this, function(ucr) {
                    // Review Question: Ist this the correct way for "default on"?
                    if (ucr['twofactor_admin/autosearch'] == null || tools.isTrue(ucr['twofactor_admin/autosearch'])) {
                        this._grid.filter(this._grid.query);
                    }
            }))
        },

        buildRendering: function () {
            this.inherited(arguments);
            this.standbyDuring(this.umcpCommand("twofactor_admin/get_is_admin", "")).then(lang.hitch(this, function (data) {
                // Admin does not need to choose a school, admin is above schools
                if (data.result.is_admin === true) {
                    this.tokenSelfService()
                } else {
                    // if theres more than one school for the user, show the chooseschool dialog first.
                    this.standbyDuring(this.umcpCommand("twofactor_admin/schools/query", "")).then(lang.hitch(this, function (ret) {
                        this._schools = ret['result']
                    })).then(lang.hitch(this, function () {
                            if (this._schools.length == 1) {
                                this._currentSchool = this._schools[0]
                                this.showTokenGrid(this._currentSchool.id)
                            } else {
                                this._choosePage = new Page({
                                    helpText: _('Select School'),
                                    fullWidth: true
                                });
                                this.addChild(this._choosePage);
                                this._choosePage.addChild(new UserGridChooseSchool({
                                    description: this.description,
                                    schools: this._schools,
                                    udmLinkEnabled: this.udmLinkEnabled,
                                    autoSearch: this.autoSearch,
                                    umcpCommand: lang.hitch(this, 'umcpCommand'),
                                    moduleFlavor: this.moduleFlavor,
                                    module: this
                                }))
                            }
                        }
                    ))
                }
            }));
        }
    });
});
